defmodule DecentralisedBookIndex.Metadata.Forms.BookFormTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Metadata

  describe "create new book via the form" do
    test "just name and description" do
      user = generate(user())
      bids = bids(actor: user)
      author_roles = author_roles(actor: user)

      valid_params = %{
        "title" => "Book",
        "description" => "A cool author",
        "format" => "Paper",
        "language" => "English",
        "page_count" => 600,
        "published" => ~D[2025-03-06],
        "author_roles" => author_roles,
        "bids" => bids
      }

      form = \
        AshPhoenix.Form.for_create(Metadata.Book, :create,
          as: "book",
          actor: user
        )

      assert {:ok, user} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end

  describe "update existing book via the form" do
    test "update name" do
      user = generate(user())
      book = generate(book())

      valid_params = %{
        "title" => "Another Book",
        "description" => "A cool book",
      }

      form =
        AshPhoenix.Form.for_update(book, :update,
          as: "book",
          actor: user
        )

      assert {:ok, user} = AshPhoenix.Form.submit(form, params: valid_params)
    end

    test "update bids and authors" do
      user = generate(user())
      book = generate(book())
      bids = bids(actor: user)
      author_roles = author_roles(actor: user)

      valid_params = %{
        "author_roles" => author_roles,
        "bids" => bids
      }

      form =
        AshPhoenix.Form.for_update(book, :update,
          as: "book",
          actor: user
        )

      assert {:ok, user} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end
end
